/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Applikationsklasse zum Löschen von Testmeldungen.
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class LoescheRDSMeldungen implements Runnable, StandardApplication {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private Debug debug;

	/**
	 * -pidPrefix von Kommandozeile.
	 */
	private String pidPrefix;

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavInterface connection;

	/**
	 * Mengenbezeichner für die dynamische Menge.
	 */
	static final String MENGE = "TMCMeldungen";

	/**
	 * Default- Konstruktor.<br>
	 * Wird typischerweise von <code>main</code> aufgerufen
	 */
	public LoescheRDSMeldungen() {
		try {
			Thread.sleep(100);
		} catch (final InterruptedException e) {
		}
	}

	/**
	 * Überschriebene Methode von <code>StandardApplication</code>, die die
	 * speziellen Startparameter auswertet.
	 * 
	 * @param argumentList
	 *            siehe
	 *            sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
	 */
	public void parseArguments(final ArgumentList argumentList) {
		debug = Debug.getLogger();

		pidPrefix = argumentList.fetchArgument("-pidPrefix=").asString();

		argumentList.fetchUnusedArguments();
	}

	/**
	 * Überschriebene Methode von StandardApplication, die die Initialisierung
	 * durchführt.
	 * 
	 * @param conn
	 *            siehe
	 *            sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	public void initialize(final ClientDavInterface conn) {
		this.connection = conn;

		new Thread(this).start();
	}

	@Override
	public void run() {
		final SystemObjectType objTyp = connection.getDataModel().getType("typ.tmcVerkehrsMeldung");
		final SystemObjectType confTyp = connection.getDataModel().getType("typ.tmcLandesMeldeStelle");
		int anzahl = 0;

		debug.info("Alle Objekte vom Typ " + objTyp.getPid() + ", deren PID mit \"" + pidPrefix
				+ "\" beginnt\n- werden gelöscht\n- entsprechende dynamische Mengen werden bereinigt");

		for (SystemObject obj : objTyp.getElementsInPeriod(1, (new Date()).getTime())) {
			if (obj.isValid() && obj.getPid().startsWith(pidPrefix)) {
				boolean fehler = false;
				final Map<SystemObject, MutableSet> bereinigteMengen = new HashMap<SystemObject, MutableSet>();

				for (SystemObject conf : confTyp.getObjects()) {
					if (conf.isValid()) {
						final MutableSet menge = ((ConfigurationObject) conf).getMutableSet(MENGE);
						if (menge != null) {
							try {
								menge.remove(obj);
								bereinigteMengen.put(conf, menge);
								debug.info("Objekt \"" + obj.getPid() + "\" bereinigt in Menge \"" + MENGE
										+ "\" von Konfigurationsobjekt " + conf.getName());
							} catch (final ConfigurationChangeException e) {
								fehler = true;
								debug.error("Objekt \"" + obj.getPid() + "\" konnte nicht bereingt werden in Menge \""
										+ MENGE + "\" von Konfigurationsobjekt " + conf.getName(), e);
								break;
							}
						}
					}
				}
				if (!fehler) {
					try {
						obj.invalidate();
						anzahl += 1;
					} catch (final ConfigurationChangeException e) {
						fehler = true;
						debug.error("Objekt \"" + obj.getPid() + "\" konnte nicht gelöscht werden", e);
					}
				}
				if (fehler) {
					for (SystemObject conf : bereinigteMengen.keySet()) {
						final MutableSet menge = bereinigteMengen.get(conf);
						try {
							menge.add(obj);
						} catch (final ConfigurationChangeException e) {
							debug.error("Bereinigte Menge \"" + MENGE + "\" des Konfigurationsobjekts " + conf.getName()
									+ "\" konnte nicht wiederhergestellt werden", e);
							continue;
						}
					}
					continue;
				}
			}
		}
		debug.info(anzahl + " Objekt(e) vom Typ " + objTyp.getPid() + " wurden erfolgreich gelöscht");
		System.exit(0);
	}

	/**
	 * Programmeinstieg.
	 * 
	 * @param arguments
	 *            Argumente
	 */
	public static void main(final String[] arguments) {
		StandardApplicationRunner.run(new LoescheRDSMeldungen(), arguments);
	}

}
