/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc;

import java.lang.Thread.UncaughtExceptionHandler;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.ste.tmc.konfiguration.Konfiguration;
import de.bsvrz.ste.tmc.meldungserstellung.LocationMethode;
import de.bsvrz.ste.tmc.meldungserstellung.MeldungenGlobal;
import de.bsvrz.ste.tmc.meldungserstellung.Meldungserstellung;
import de.bsvrz.ste.tmc.meldungserstellung.PublikationInitial;
import de.bsvrz.ste.tmc.parameter.RDSParameterVerwaltung;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Applikationsklasse der SWE TMC-Meldungen.<br>
 * Für das übergebene Netz und die übergebene LMS wird eine Applikations-
 * Instanz der SWE TMC-Meldungen erzeugt und initialisiert.<br>
 * SWE-spezifische Parameter:<br>
 * -netz: Das Netz, dessen Infrastrukturobjekte (Strassensegmente,
 * Messquerschnitte etc.) betrachtet werden. Da ein Netz weitere Netze enthalten
 * kann, erfolgt die Auswertung rekursiv.<br>
 * -lms: Die Landesmeldestelle, auf welche diese Instanz der SWE sich bezieht.
 * Das Konfigurationsobjekt der LMS enthält insbesondere die zu verwendenden
 * RDS-Modell-Parameter.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class TMCApp implements StandardApplication, MeldungenGlobal {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private Debug debug;

	/**
	 * -netz von Kommandozeile.
	 */
	private String netzPid;

	/**
	 * -lms von Kommandozeile.
	 */
	private String lmsPid;

	/**
	 * -konfigBereich von Kommandozeile.
	 */
	private String konfigBereichPid;

	/**
	 * -locationMethode von Kommandozeile.
	 */
	private int locationMethodeId;

	/**
	 * -publikationInitial von Kommandozeile.
	 */
	private int publikationInitialId;

	/**
	 * -idOrganisation von Kommandozeile.
	 */
	private String idOrganisation;

	/**
	 * -idNummer von Kommandozeile.
	 */
	private String idNummer;

	/**
	 * -ablaufOffset von Kommandozeile.
	 */
	private int ablaufOffset;

	private ConfigurationArea konfigBereich;

	private LocationMethode locationMethode;

	private PublikationInitial publikationInitial;

	private ClientDavInterface dav;

	private Meldungserstellung meldungserstellung;

	/**
	 * Timeout in ms für Daten der Parametierung.
	 */
	public static final long TIMEOUT = 5000;

	/**
	 * Default- Konstruktor.<br>
	 * Wird typischerweise von <code>main</code> aufgerufen
	 */
	public TMCApp() {
		try {
			Thread.sleep(100);
		} catch (final InterruptedException e) {
		}
	}

	/**
	 * Extrahiert das Konfigurationsobjekt der übergebenenen PID vom Typ
	 * <code>typ.netz</code> und führt ggf. eine Typprüfung durch.
	 * 
	 * @param pid
	 *            die PID des Konfigurationsobjektes, das extrahiert werden soll
	 * @param typ
	 *            der Typ, den das Konfigutationsobjekt haben muss - wenn hier
	 *            <code>null</code> übergeben wird, unterbleibt eine Typprüfung.
	 * @return das gefundene Konfigurationsobjekt
	 * @throws IllegalArgumentException
	 *             falls Konfigurationsobjekt nicht gefunden oder vom falschen
	 *             Typ
	 */
	private ConfigurationObject getKonfigObjekt(final String pid, final String typ) {
		ConfigurationObject obj = null;

		try {
			obj = (ConfigurationObject) dav.getDataModel().getObject(pid);
		} catch (final ClassCastException e) {
			throw new IllegalArgumentException("Die PID \"" + pid + "\" bezeichnet kein KonfigurationsObjekt");
		}

		if (obj == null) {
			throw new IllegalArgumentException("Zur PID \"" + pid + "\" wurde kein Konfigurationsobjekt gefunden ");
		}

		if (typ != null && !obj.isOfType(typ)) {
			throw new IllegalArgumentException(
					"Das Konfigurationsobjekt \"" + obj.getPid() + "\" ist nicht vom Typ " + typ);
		}

		return obj;
	}

	/**
	 * Extrahiert - nach Prüfung - den Konfigurationsbereich aus dem Attribut
	 * {@link #konfigBereich}.
	 * 
	 * @param pid
	 *            PID
	 * 
	 * @return den gefunden Konfigurationsbereich
	 * @throws IllegalArgumentException
	 *             falls Konfigurationsbereich nicht gefunden
	 */
	private ConfigurationArea getKonfigBereich(final String pid) {
		ConfigurationArea ca = null;
		if (pid != null && !"".equals(pid)) {
			ca = dav.getDataModel().getConfigurationArea(pid);
			if (ca == null) {
				throw new IllegalArgumentException(
						"Zur PID \"" + pid + "\" wurde kein Konfigurationsbereich gefunden ");
			}
			return ca;
		} else {
			final DataModel dm = dav.getDataModel();
			final ConfigurationObject co = dm.getConfigurationAuthority();
			final AttributeGroup atg = dm.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
			final Data data = co.getConfigurationData(atg);
			if (data != null) {
				final TextArray array = data.getTextArray("defaultBereich");
				if (array.getLength() > 0) {
					ca = (ConfigurationArea) dm.getObject(array.getText(0));
				}
			}
			if (ca == null) {
				throw new IllegalArgumentException("Es konnte kein Default-Konfigurationsbereich bestimmt werden");
			}
		}
		return ca;
	}

	private LocationMethode getLocationMethode(final int id) {
		final LocationMethode methode = LocationMethode.get(id);
		if (methode == null) {
			throw new IllegalArgumentException("LocationMethode \"" + id + "\" wird nicht unterstützt");
		}
		return methode;
	}

	private PublikationInitial getPublikationInitial(final int id) {
		final PublikationInitial pub = PublikationInitial.get(id);
		if (pub == null) {
			throw new IllegalArgumentException("PublikationInitial \"" + id + "\" wird nicht unterstützt");
		}
		return pub;
	}

	/**
	 * Programmeinstieg TMC.<br>
	 * 
	 * @param arguments
	 *            Kommandozeilenargumente, folgende speziellen Argumente werden
	 *            ausgewertet:
	 *            <p>
	 *            -netz: Netz, dessen Infrastrukturobjekte verwendet werden.
	 *            -lms: Zuständige Landesmeldestelle (LMS).
	 */
	public static void main(final String[] arguments) {
		if (uehDebug == null) {
			uehDebug = Debug.getLogger();
		}

		Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler() {
			public void uncaughtException(final Thread t, final Throwable e) {
				uehDebug.error("Uncaught Exception " + e + "in Thread " + t + ":\n", e);
			}
		});

		StandardApplicationRunner.run(new TMCApp(), arguments);
	}

	/**
	 * Debug-Logger für Logging-Ausgaben des UncaughtExceptionHandlers.
	 */
	private static Debug uehDebug;

	/***************************************************************************/
	/* Implementierung Interface StandardApplication */
	/***************************************************************************/

	/**
	 * Überschriebene Methode von <code>StandardApplication</code>, die die
	 * speziellen Startparameter auswertet.<br>
	 * Die Argumtwerte, die ausgewertet werden, sind durch die Attribute
	 * {@link #netzPid} und {@link #lmsPid} spezifiziert
	 * 
	 * @param argumentList
	 *            siehe
	 *            sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
	 */
	public void parseArguments(final ArgumentList argumentList) {
		debug = Debug.getLogger();

		// Pflichtargumete
		netzPid = argumentList.fetchArgument("-netz=").asNonEmptyString();
		lmsPid = argumentList.fetchArgument("-lms=").asNonEmptyString();

		// Optionale Argumente
		konfigBereichPid = argumentList.fetchArgument("-konfigBereich=").asString();
		locationMethodeId = argumentList.fetchArgument("-locationMethode=0").intValueBetween(0, 1);
		publikationInitialId = argumentList.fetchArgument("-publikationInitial=0").intValueBetween(0, 1);

		idOrganisation = argumentList.fetchArgument("-idOrganisation=").asString();
		idNummer = argumentList.fetchArgument("-idNummer=").asString();
		ablaufOffset = argumentList.fetchArgument("-ablaufOffset=120").intValueBetween(0, 144000);

		argumentList.fetchUnusedArguments();
	}

	/**
	 * Überschriebene Methode von StandardApplication, die die Initialisierung
	 * durchführt.<br>
	 * Es wird eine Instanz von <code>VerwaltungEngstellen</code> erzeugt und
	 * initialisiert
	 * 
	 * @param connection
	 *            siehe
	 *            sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	public void initialize(final ClientDavInterface connection) {
		dav = connection;

		Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler() {
			public void uncaughtException(final Thread t, final Throwable e) {
				debug.error("Uncaught Exception " + e + "in Thread " + t + ":\n", e);
			}
		});

		try {
			MessageSender.getInstance().setApplicationLabel(connection.getLocalApplicationObject().getName());

			// Initialisiere die Objekte der Daten des Interfaces
			// MeldungenGlobal
			locationMethode = getLocationMethode(locationMethodeId);
			publikationInitial = getPublikationInitial(publikationInitialId);
			konfigBereich = getKonfigBereich(konfigBereichPid);
			debug.config(
					"Neue dynamische Objekte werden im Konfigurationsbereich " + konfigBereich.getPid() + " abgelegt");

			// Bestimme benötigte Konfigurationsobjekte
			final ConfigurationObject netz = getKonfigObjekt(netzPid, "typ.netz");
			final ConfigurationObject lms = getKonfigObjekt(lmsPid, "typ.tmcLandesMeldeStelle");

			// Initialisiere Konfigurationsobjekte des/der Netzes/Netze
			final Konfiguration konfiguration = new Konfiguration(connection, netz, true, true);

			// Initialisiere Verwaltung Modell-Parameter der LMS
			final RDSParameterVerwaltung parameterVerwaltung = new RDSParameterVerwaltung(connection, lms);
			if (!parameterVerwaltung.wartenAufParameter(TIMEOUT)) {
				throw new IllegalStateException("Keine Parameterdaten empfangen nach " + TIMEOUT + "ms");
			}

			// instanziiere Meldungserstellung
			meldungserstellung = new Meldungserstellung(connection, konfiguration, parameterVerwaltung, this, false);

			debug.info("SWE TMC-Meldungen initialisiert");
		} catch (final Exception e) {
			debug.error("Abbruch der SWE TMC-Meldungen während der Initialisierung", e);
			System.exit(1);
		}
	}

	/***************************************************************************/
	/* Implementierung Interface MeldungenGlobal */
	/***************************************************************************/

	/**
	 * @return den Konfigurationsbereich, in dem dynamische Meldungsobjekte
	 *         abgelegt werden
	 */
	public ConfigurationArea getKonfigurationsbereich() {
		return konfigBereich;
	}

	/**
	 * @return die LocationMethode, nach der Meldungsdatensätze kodiert werden
	 */
	public LocationMethode getLocationMethode() {
		return locationMethode;
	}

	/**
	 * @return die Methode, nach der beim Programmstart aktuelle Zustände
	 *         publiziert werden
	 */
	public PublikationInitial getPublikationInitial() {
		return publikationInitial;
	}

	/**
	 * @return die (Text-) Id der Organisationseinheit, die einer generierten
	 *         Meldung zugeordnet sein soll - i. d. R. LMS
	 */
	public String getIdOrganisation() {
		return idOrganisation;
	}

	/**
	 * @return die (numerische) Id der Organisationseinheit, die einer
	 *         generierten Meldung zugeordnet sein soll - i. d. R. LMS
	 */
	public String getIdNummer() {
		return idNummer;
	}

	/**
	 * @return Zeitspanne in min., wielange eine Meldung gültig (nicht
	 *         abgelaufen) sein soll, nachdem sie aktualisiert wurde
	 */
	public int getAblaufOffset() {
		return ablaufOffset;
	}
}
