/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.junit;

import static org.junit.Assert.assertTrue;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.ste.tmc.allgemein.Beschleunigung;
import de.bsvrz.ste.tmc.junit.tools.JUnitHelper;
import de.bsvrz.ste.tmc.junit.tools.ResultWrapper;
import de.bsvrz.ste.tmc.meldungserstellung.VerwaltungMeldungen;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Statische Definitionen zur Benutzung in den JUnit-Tests.
 * 
 * @author Sans
 *
 */
public class RDSTMCPrueffall {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug _debug = Debug.getLogger();

	protected static final long KORREKTUR = 100;

	protected static final long TOL_PLUS = 15000;
	protected static final long TOL_MINUS = 5000;

	protected static final String ATG_PID = "atg.tmcVerkehrsMeldung";
	protected static final String ASP_PID = "asp.tmcGeneriert";

	private static long _startZeitPunkt;

	protected static long getStartZeitpunkt() {
		if (_startZeitPunkt == 0) {
			throw new UnsupportedOperationException("Startzeitpunkt ist nicht initialisiert");
		}
		return _startZeitPunkt;
	}

	protected static void setStartZeitpunkt(final long startZeitPunkt) {
		if (_startZeitPunkt > 0) {
			throw new UnsupportedOperationException("Startzeitpunkt ist bereits initialisiert");
		}
		if (startZeitPunkt <= 0) {
			throw new IllegalArgumentException("Startzeitpunkt muß größer als 0 sein");
		}
		_startZeitPunkt = startZeitPunkt;
	}

	protected static void setTestGenerator(final Thread testGenerator) {
		_testGenerator = testGenerator;
	}

	private static Thread _testGenerator;

	protected static void assertTestGeneratorLaeuft() {
		assertTrue("Testgenerator muss laufen", _testGenerator != null && _testGenerator.isAlive());
	}

	protected static void assertPruefZeitKorrekt(final long pruefZeitSoll, final long pruefZeitIst) {
		final long relPruefZeit = (pruefZeitIst - getStartZeitpunkt()) * Beschleunigung.WERT;
		_debug.config("Zeitpunkt des Prüfereignisses: " + relPruefZeit + "ms");
		assertTrue(
				"Zeitpunkt des Prüfereignisses zwischen " + (pruefZeitSoll - TOL_MINUS) + "ms" + " und "
						+ (pruefZeitSoll + TOL_PLUS) + "ms",
				relPruefZeit >= (pruefZeitSoll - TOL_MINUS) && relPruefZeit <= (pruefZeitSoll + TOL_PLUS));
	}

	protected static void assertMeldungsDatenKorrekt(final ResultData rd, final int informationsTyp,
			final String meldungsArt, final int geografischeBedeutung, final int zeitlicheDringlichkeit,
			final int zeitlicheDauer, final int ausmassRichtung, final int ereignisCode, final int primaereLocation,
			final String optionalKennung, final String optionalWert) {
		final ResultWrapper rw = new ResultWrapper(rd);

		// assertTrue(
		// "InformationsTyp hat Wert " + informationsTyp
		// + (informationsTyp == InformationsTyp.ALERTC ? " (für Alert-C)" :
		// ""),
		// rw.getInformationsTyp() == informationsTyp);

		assertTrue("Meldungsart ist " + meldungsArt, rw.getMeldungsArt().equals(meldungsArt));

		// assertTrue(
		// "Geografische Bedeutung hat Wert " + geografischeBedeutung
		// + (geografischeBedeutung == GeografischeBedeutung.NATIONAL ? " (für
		// national)" : ""),
		// rw.getGeografischeBedeutung() == GeografischeBedeutung.NATIONAL);

		assertTrue(
				"Zeitliche Dringlichkeit hat Wert " + zeitlicheDringlichkeit
						+ (zeitlicheDringlichkeit == ZeitlicheDringlichkeit.NORMAL ? " (für normal)" : ""),
				rw.getZeitlicheDringlichkeit() == zeitlicheDringlichkeit);

		// assertTrue(
		// "Zeitliche Dauer hat Wert " + zeitlicheDauer
		// + (zeitlicheDauer == ZeitlicheDauer.UNBEKANNT ? " (für unbekannt)" :
		// ""),
		// rw.getZeitlicheDauer() == zeitlicheDauer);
		//
		// assertTrue("Ausmaß-/Richtung ist " + ausmassRichtung,
		// rw.getAusmassRichtung() == ausmassRichtung);

		assertTrue("Ereigniscode ist " + ereignisCode, rw.getEreignisCode() == ereignisCode);

		assertTrue("Primaere TMC-Location ist " + primaereLocation, rw.getPrimaereLocation() == primaereLocation);

		// if (optionalKennung != null) {
		// assertTrue("Wert mit (optionaler) Kennung " + optionalKennung + " ist
		// " + optionalWert,
		// rw.getOptionalWert(optionalKennung).equals(optionalWert));
		// }
	}

	protected static MutableSet getMeldungsMenge() {
		return JUnitHelper.getInstance().getLms().getMutableSet(VerwaltungMeldungen.MENGE);
	}

	protected static ClientDavInterface getConnection() {
		return JUnitHelper.getInstance().getConnection();
	}

	protected static DataModel getDataModel() {
		return getConnection().getDataModel();
	}

	private static DataDescription _dataDescription = new DataDescription(getDataModel().getAttributeGroup(ATG_PID),
			getDataModel().getAspect(ASP_PID));

	public static DataDescription getDataDescription() {
		return _dataDescription;
	}

	public static class InformationsTyp {
		public static final int ALERTC = 0;
	}

	public static class MeldungsArt {
		public static final String AKTUALISIERUNG = "Geändert";
		public static final String AUFHEBUNG = "Gelöscht";
	}

	public static class GeografischeBedeutung {
		public static final int NATIONAL = 3;
	}

	public static class ZeitlicheDringlichkeit {
		public static final int NORMAL = 0;
	}

	public static class ZeitlicheDauer {
		public static final int UNBEKANNT = 0;
	}

}
