/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.konfiguration;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein Strassenknoten modelliert.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class StrassenKnoten extends DavProxyObjekt {
	/**
	 * Debug-Logger f&uuml;r Logging-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Erstellt eine neue Instanz vom Typ <code>StrassenKnoten</code>.
	 * 
	 * @param davObjekt
	 *            das zuzuordnente Dav-Objekt
	 * @param typ
	 *            Typ
	 */
	public StrassenKnoten(final SystemObject davObjekt, final String typ) {
		super(davObjekt);
		this.typ = typ;
	}

	/**
	 * L&auml;nge in m dieses Stra&szlig;ensegments.
	 */
	private String typ;

	/**
	 * @return die L&auml;nge in m dieses Stra&szlig;ensegments
	 */
	public String getTyp() {
		return typ;
	}

	/**
	 * Die von diesem Strassenknoten abgehenden &auml;u&szlig;eseren
	 * Strassensegmente.
	 */
	private List<AeusseresStrassenSegment> abgehendeAeussereSegmente = new Vector<AeusseresStrassenSegment>();

	/**
	 * Read-Only-Zugriff auf die von diesem Strassenknoten abgehenden äusseren
	 * Strassensegmente
	 * 
	 * @return Liste der von diesem Strassenknoten abgehenden äusseren
	 *         Strassensegmente
	 */
	public List<AeusseresStrassenSegment> getAbgehendeSegmente() {
		return Collections.unmodifiableList(abgehendeAeussereSegmente);
	}

	/**
	 * Fügt diesem Knoten ein abgehendes Segment hinzu
	 * 
	 * @param segment
	 *            das neue abgehende Segment
	 */
	void addAbgehendesSegment(AeusseresStrassenSegment segment) {
		abgehendeAeussereSegmente.add(segment);
	}

	/**
	 * Die zu diesem Strassenknoten hinf&uuml;hrenden &auml;u&szlig;eren
	 * Strassensegmente.
	 */
	private List<AeusseresStrassenSegment> hinfuehrendeAeussereSegmente = new Vector<AeusseresStrassenSegment>();

	/**
	 * Read-Only-Zugriff auf die zu diesem Strassenknoten hinführenden äusseren
	 * Strassensegmente
	 * 
	 * @return Liste der zu diesem Strassenknoten hinführenden äusseren
	 *         Strassensegmente
	 */
	public List<AeusseresStrassenSegment> getHinfuehrendeSegmente() {
		return Collections.unmodifiableList(hinfuehrendeAeussereSegmente);
	}

	/**
	 * Fügt diesem Knoten ein hinführendes Segment hinzu
	 * 
	 * @param segment
	 *            das neue hinführende Segment
	 */
	void addHinfuehrendesSegment(AeusseresStrassenSegment segment) {
		hinfuehrendeAeussereSegmente.add(segment);
	}

	/**
	 * Die zu diesem Strassenknoten geh&ouml;renden inneren Strassensegmente.
	 */
	private List<InneresStrassenSegment> innereSegmente = new Vector<InneresStrassenSegment>();

	/**
	 * Read-Only-Zugriff auf die zu diesem Strassenknoten gehörenden inneren
	 * Strassensegmente
	 * 
	 * @return Liste der zu diesem Strassenknoten gehörenden inneren
	 *         Strassensegmente
	 */
	public List<InneresStrassenSegment> getInnereSegmente() {
		return Collections.unmodifiableList(innereSegmente);
	}

	/**
	 * Fügt diesem Knoten ein inneres Segment hinzu
	 * 
	 * @param segment
	 *            das neue hinführende Segment
	 */
	void addInneresSegment(InneresStrassenSegment segment) {
		innereSegmente.add(segment);
	}

	/**
	 * Methode prüft ob es sich bei dem Knoten um ein Autobahnkreuz handelt
	 * 
	 * @return Knoten ist Autobahnkreuz: true, sonst false
	 */
	public boolean isAutobahnKreuz() {
		return typ.equals("AutobahnKreuz");
	}

	/**
	 * Methode prüft ob es sich bei dem Knoten um ein Autobahndreieck handelt
	 * 
	 * @return Knoten ist Autobahndreieck: true, sonst false
	 */
	public boolean isAutobahnDreieck() {
		return typ.equals("AutobahnDreieck");
	}

	/**
	 * Methode prüft ob es sich bei dem Knoten um ein Autobahnende handelt
	 * 
	 * @return Knoten ist Autobahnende: true, sonst false
	 */
	public boolean isAutobahnEnde() {
		return typ.equals("AutobahnEnde");
	}

	/**
	 * Methode prüft ob es sich bei dem Knoten um einen Autobahnanfang handelt
	 * 
	 * @return Knoten ist Autobahnanfang: true, sonst false
	 */
	public boolean isAutobahnAnfang() {
		return typ.equals("AutobahnAnfang");
	}

	/**
	 * Methode prüft ob es sich bei dem Knoten um eine Autobahnanschlussstelle
	 * handelt
	 * 
	 * @return Knoten ist eine Autobahnanschlussstelle: true, sonst false
	 */
	public boolean isAutobahnAnschlussStelle() {
		return typ.equals("AutobahnAnschlussStelle");
	}

	/**
	 * Methode pr&uuml;ft ob es sich bei dem Knoten um einen sonstigen Knoten
	 * handelt.
	 * 
	 * @return Knoten ist ein sonstiger Knoten: true, sonst false
	 */
	public boolean isSonstigerKnoten() {
		return "SonstigerKnoten".equals(typ);
	}
}
