/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * WeiÃŸenfelser StraÃŸe 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.meldungserstellung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.tmc.allgemein.RDSEreignis;
import de.bsvrz.ste.tmc.konfiguration.TmcLocationPunkt;
import de.bsvrz.ste.tmc.parameter.ParameterSender;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse zum Versenden von Daten unter dem Aspekt "TMCSenden" der
 * Attributgruppe "TMCVerkehrsmeldung". Sie implementiert das
 * ClientSenderInterface welches die Methoden zum Versenden der Daten
 * bereitstellt.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class Meldung implements ClientSenderInterface {

	public static final String TMCMELDUNG_ATG = "atg.tmcVerkehrsMeldung";

	public static final String TMCMELDUNG_ASP_GENERIERT = "asp.tmcGeneriert";

	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug _debug = Debug.getLogger();

	/**
	 * Ist auf true gesetzt, solange auf Sendesteuerung gewartet wird.
	 */
	private boolean _warten = false;

	/**
	 * Rolle, mit der als Sender angemeldet wird.
	 */
	public static final SenderRole ROLLE = SenderRole.source();

	/**
	 * Zeitin ms, die max. auf Sendesteuerung gewartet wird.
	 */
	public static final long SENDE_TIMEOUT = 5000;

	/**
	 * Ist diese Instanz zum DAV-Senden angemeldet?
	 */
	private boolean _angemeldet = false;

	/**
	 * @return ist diese Instanz zum DAV-Senden angemeldet?
	 */
	public boolean isAngemeldet() {
		return _angemeldet;
	}

	private VerwaltungMeldungen _verwaltung;

	protected VerwaltungMeldungen getVerwaltung() {
		return _verwaltung;
	}

	/**
	 * @return die Datenverteiler-Verbindung
	 */
	public ClientDavInterface getConnection() {
		return getVerwaltung().getConnection();
	}

	private SystemObject _davObjekt;

	/**
	 * @return das Dav-Systemobject, für das das Senden von Daten erfolgt
	 */
	public SystemObject getDavObjekt() {
		return _davObjekt;
	}

	/**
	 * Konstruktor der Klasse.
	 * 
	 * @param verwaltung
	 *            {@link VerwaltungMeldungen}
	 * @param davObjekt
	 *            Objekt
	 * @param datenGeneriert
	 *            {@link ResultData}
	 */
	public Meldung(final VerwaltungMeldungen verwaltung, final SystemObject davObjekt,
			final ResultData datenGeneriert) {
		_verwaltung = verwaltung;
		_davObjekt = davObjekt;
		_datenGeneriert = datenGeneriert;

		if (datenGeneriert != null) {
			_erzeugungsZeit = getErzeugungsZeit(_datenGeneriert.getData());
		}

		try {
			getConnection().subscribeSender(this, getDavObjekt(), getVerwaltung().getDatenBeschreibung(), ROLLE);
			_debug.finer(getVerwaltung().getDatenBeschreibung() + " zum Senden angemeldet für " + getDavObjekt());
			_angemeldet = true;
		} catch (final OneSubscriptionPerSendData e) {
			/*
			 * Es liegt bereits eine Sendeanmeldung von anderer Stelle vor =>
			 * hier nicht anmnelden
			 */
			_debug.warning(getDavObjekt().getPid() + " ist bereits an anderer Stelle zum Senden angemeldet", e);
		}
	}

	public Meldung(final VerwaltungMeldungen verwaltung, final SystemObject davObjekt) {
		this(verwaltung, davObjekt, null);
	}

	/**
	 * Gibt die mit dieser Instanz verbundenen Ressourcen wieder frei.
	 */
	public void dispose() {
		if (_angemeldet) {
			getConnection().unsubscribeSender(this, getDavObjekt(), getVerwaltung().getDatenBeschreibung());
			_debug.config("Senden abgemeldet von " + getVerwaltung().getDatenBeschreibung() + " für " + getDavObjekt());
			_angemeldet = false;
		}
	}

	protected boolean send(final Ereignis ereignis) {
		final ResultData resultData = getResultData(ereignis);
		if (send(resultData)) {
			new ParameterSender(getConnection(), getDavObjekt(), resultData, TMCMELDUNG_ATG).start();
			_datenGeneriert = resultData;
			// _version = getVersion(_datenGeneriert.getData());
			_erzeugungsZeit = getErzeugungsZeit(_datenGeneriert.getData());

			return true;
		} else {
			return false;
		}
	}

	/**
	 * Unmittelbares Senden von Daten.
	 * 
	 * @param resultData
	 *            die zu sendenden Daten
	 * @return true, wenn das Senden erfolgreich war, false sonst
	 */
	boolean send(final ResultData resultData) {
		try {
			getConnection().sendData(resultData);
		} catch (final SendSubscriptionNotConfirmed e) {
			if (SENDE_TIMEOUT > 0) {
				_debug.config("Warte max. " + SENDE_TIMEOUT + "ms auf Sendesteuerung für " + getDavObjekt().getPid());
				synchronized (this) {
					_warten = true;
					try {
						wait(SENDE_TIMEOUT);
					} catch (final InterruptedException ex) {
						// tue nichts
					}
					_warten = false;
				}
				try {
					getConnection().sendData(resultData);
				} catch (final SendSubscriptionNotConfirmed ex) {
					_debug.warning("Keine Sendesteuerung für " + getDavObjekt().getPid(), ex);
					return false;
				}
				return true;
			}
		} catch (final DataNotSubscribedException e) {
			_debug.error("Keine Sendeanmeldung bzw. Sendeanmeldung ungültig für " + getDavObjekt().getPid(), e);
			return false;
		}
		return true;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.
	 * main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription, byte)
	 */
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		_debug.finer("dataRequest() für " + getDavObjekt().getPid() + " - Sender state: " + state);
		if (_warten && state == ClientSenderInterface.START_SENDING) {
			synchronized (this) {
				notify();
			}
		}
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.
	 * dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription)
	 */
	public boolean isRequestSupported(final SystemObject so, final DataDescription dd) {
		return true;
	}

	/**
	 * Aktuell unter dem Aspekt generiert gesendete Daten.
	 */
	private ResultData _datenGeneriert = null;

	/**
	 * @return die unter dem Aspekt senden aktuell gesendeten Daten
	 */
	protected ResultData getDatenGeneriert() {
		return _datenGeneriert;
	}

	long _erzeugungsZeit = 0;

	/**
	 * Bildet eine sendefertige ResultData-Instanz für das aktuelle
	 * Meldungsobjekt mit dem Zeitstempel der aktuellen Zeit und einem
	 * DAV-Datensatz für das übergebene Ereignis.
	 * 
	 * @param ereignis
	 *            das Ereignis, für das der die Daten erzeugt werden sollen
	 * 
	 * @return ResultData Instanz
	 */
	private ResultData getResultData(final Ereignis ereignis) {
		final Data data = getData(ereignis);
		final AttributeGroup atg = getConnection().getDataModel().getAttributeGroup(TMCMELDUNG_ATG);
		final Aspect asp = getConnection().getDataModel().getAspect(TMCMELDUNG_ASP_GENERIERT);
		return new ResultData(getDavObjekt(), new DataDescription(atg, asp), getConnection().getTime(), data);
	}

	/**
	 * Provisorischer Wert für "undefinierte" Zeitangabe.
	 */
	public static final long NULLZEIT = 31532400000L;

	/**
	 * Bildet einen DAV-Datensatz zur vorliegenden Meldung und initialisiert die
	 * relevanten Attribute bezgl. Location(s) und Ereignis
	 * 
	 * @param ereignis
	 *            das Ereignis, für das der Datensatz gebildet werden soll
	 * 
	 * @return den gebildeten DAV-Datensatz
	 */
	private Data getData(final Ereignis ereignis) {
		final long aktuelleZeit = getConnection().getTime();
		final long ablaufZeit = getVerwaltung().getAblaufOffset() > 0
				? aktuelleZeit + getVerwaltung().getAblaufOffset() * 60 * 1000 : NULLZEIT;

		final AttributeGroup atg = getConnection().getDataModel().getAttributeGroup(TMCMELDUNG_ATG);
		final Data data = getConnection().createData(atg);

		String freiText = "";
		final SystemObject ek = ereignis.getErklaerungsKomponente();
		if (ek != null) {
			freiText = "Referenzierte StÃ¶rungsursache: " + ek.getName() + " (pid: " + ek.getPid() + ")";
		} else {
			freiText = ereignis.toString();
		}
		data.getItem("Beschreibung").asTextValue().setText(freiText);

		final Data iditem = data.getItem("TMCID");
		iditem.getItem("Datenproduzent").asTextValue()
				.setText(getVerwaltung().getIdOrganisation() == null ? "" : getVerwaltung().getIdOrganisation());
		iditem.getItem("Datenkennung").asTextValue()
				.setText(getVerwaltung().getIdNummer() == null ? "" : getVerwaltung().getIdNummer());

		final Data tmcdata = data.getItem("TMCDaten");

		final Data dataVerwaltung = tmcdata.getItem("VerwaltungsInformationen");

		Array array = dataVerwaltung.getArray("ErzeugungsZeit");
		array.setLength(1);
		array.getItem(0).asTimeValue().setMillis(aktuelleZeit);

		array = dataVerwaltung.getArray("AktualisierungsZeit");
		array.setLength(1);
		array.getItem(0).asTimeValue().setMillis(aktuelleZeit);

		array = dataVerwaltung.getArray("StartZeit");
		array.setLength(1);
		array.getItem(0).asTimeValue().setMillis(aktuelleZeit);

		array = dataVerwaltung.getArray("EndeZeit");
		array.setLength(1);
		array.getItem(0).asTimeValue()
				.setMillis(ereignis.getRDSEreignis() == RDSEreignis.KEINE_AUSSAGE ? aktuelleZeit : ablaufZeit);

		_debug.fine(_davObjekt.getPid() + ", Ereignis: " + ereignis.getRDSEreignis().toString());
		dataVerwaltung.getItem("TmcStatus").asTextValue()
				.setText(ereignis.getRDSEreignis() == RDSEreignis.KEINE_AUSSAGE ? "Gelöscht" : "Geändert");

		dataVerwaltung.getItem("TmcPrioritaet").asUnscaledValue().set(0); // 0
																			// ==
																			// "Normal"
		dataVerwaltung.getItem("BearbeitungsZustand").asTextValue().setText("nicht quittiert");
		dataVerwaltung.getItem("Erzeugungsart").asTextValue().setText("automatisch");

		final Data dataVerortung = tmcdata.getItem("VerortungsInformationen");
		final Array dataLocationDaten = dataVerortung.getArray("LokationsDaten");
		dataLocationDaten.setLength(1);
		final Data locDaten = dataLocationDaten.getItem(0);

		locDaten.getItem("TMCRichtung").asUnscaledValue().set(ereignis.getTmcRichtung());

		locDaten.getItem("StartOffset").asUnscaledValue().set(0);
		locDaten.getItem("EndOffset").asUnscaledValue().set(0);

		final int size = ereignis.getTmcOrte().size();

		if (size > 0) {
			final TmcLocationPunkt ptmcLocationPunkt = ereignis.getTmcOrte().get(0);
			locDaten.getItem("PrimaerLokation").asScaledValue().set(ptmcLocationPunkt.getLocationCode());

			if (size > 1) {
				final TmcLocationPunkt stmcLocationPunkt = ereignis.getTmcOrte().get(size - 1);
				locDaten.getItem("SekundaerLokation").asScaledValue().set(stmcLocationPunkt.getLocationCode());
			}
		}

		final Data dataEreignis = tmcdata.getItem("EreignisInformationen");
		final Array dataEreignisDaten = dataEreignis.getArray("EreignisDaten");

		dataEreignisDaten.setLength(1);

		final Data ereignisDaten = dataEreignisDaten.getItem(0);
		ereignisDaten.getItem("EreignisCode").asTextValue().setText(ereignis.getEreignisCode());
		ereignisDaten.getItem("EmpfehlungsCode").asUnscaledValue().set(0);
		ereignisDaten.getItem("VorhersageCode").asUnscaledValue().set(0);

		return data;
	}

	private static long getErzeugungsZeit(final Data data) {

		final Data tmcdata = data.getItem("TMCDaten");

		final Data dataVerwaltung = tmcdata.getItem("VerwaltungsInformationen");

		final Array array = dataVerwaltung.getArray("ErzeugungsZeit");
		return array.getItem(0).asTimeValue().getMillis();
	}

}
