/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.test;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Empfangen und Parameterdatensatzes.
 * 
 * Die Empfangs-Anmeldung -und Abmeldung erfolgt im Konstruktor.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:25 $ / ($Author: Sans $)
 */
public class ParameterEmpfaenger implements ClientReceiverInterface {

	/**
	 * Debug-Logger für Logging-Ausgaben
	 */
	private final static Debug _debug = Debug.getLogger();

	/**
	 * Zeitin ms, die max. auf Sendesteuerung gewartet wird
	 */
	static final long TIMEOUT = 5000;

	/**
	 * Die Datenverteilerverbindung
	 */
	final ClientDavInterface connection;

	/**
	 * Das DAV-Objekt, für das das Senden erfolgt
	 */
	final SystemObject davObjekt;

	/**
	 * Datenbeschreibung der Daten, die aus der Parametrierung gelesen werden
	 */
	final DataDescription datenBeschreibung;

	/**
	 * Ist auf true gesetzt, solange auf update gewartet wird
	 */
	private boolean _warten = false;

	/**
	 * Empfangene Daten
	 */
	private Data _data = null;

	public ParameterEmpfaenger(ClientDavInterface connection, SystemObject davObjekt,
			DataDescription datenBeschreibung) {
		this.connection = connection;
		this.davObjekt = davObjekt;
		this.datenBeschreibung = datenBeschreibung;

		connection.subscribeReceiver(this, davObjekt, datenBeschreibung, ReceiveOptions.normal(),
				ReceiverRole.receiver());
		_debug.finest(datenBeschreibung + " zum Empfang angemeldet für " + davObjekt);

		_debug.finer("Warte max. " + TIMEOUT + "ms auf Update Parameter-Daten von "
				+ datenBeschreibung.getAttributeGroup().getPid() + " für " + davObjekt.getPid());
		synchronized (this) {
			if (getData() == null) {
				_warten = true;
				try {
					wait(TIMEOUT);
				} catch (InterruptedException e) {
					// tue nichts
				}
				_warten = false;
			}
		}

		if (getData() == null) {
			_debug.warning("Keine Parameter-Daten empfangen von " + datenBeschreibung.getAttributeGroup().getPid()
					+ " für " + davObjekt.getPid());
		}

		connection.unsubscribeReceiver(this, davObjekt, datenBeschreibung);
		_debug.finest("Empfang abgemeldet von " + datenBeschreibung + " für " + davObjekt);
	}

	/**
	 * @return die empfangenen Daten
	 */
	public Data getData() {
		return _data;
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.
	 * main.ResultData[])
	 */
	public void update(ResultData[] resultData) {
		for (ResultData data : resultData) {
			Aspect asp = data.getDataDescription().getAspect();
			AttributeGroup atg = data.getDataDescription().getAttributeGroup();

			if (atg.equals(datenBeschreibung.getAttributeGroup()) && asp.equals(datenBeschreibung.getAspect())) {
				if (data.hasData()) {
					_data = data.getData();
				}
			}
		}
		if (_data != null && _warten) {
			synchronized (this) {
				notify();
			}
		}
	}

}
